# Copyright (c) 2018 NTT DATA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs
from masakariclient import api_versions
from masakariclient import plugin

from masakaridashboard.api import api
from masakaridashboard.notifications import tables as notification_tab


class OverviewTab(tabs.Tab):
    name = _("Notifications")
    slug = "notifications"
    template_name = ("masakaridashboard/notifications/_detail_overview.html")

    def get_context_data(self, request):
        return {"notification": self.tab_group.kwargs['notification']}


class NotificationProgressDetailsTab(tabs.TableTab):
    table_classes = (notification_tab.NotificationProgressDetailsTable,)
    name = _("Progress Details")
    slug = "notification_progress_details"
    template_name = "masakaridashboard/notifications/_progress_detail.html"
    preload = False

    def get_notification_progress_details_data(self):
        try:
            id = 0
            notification = self.tab_group.kwargs['notification']
            notification_obj = \
                api.get_notification_with_progress_details(
                    self.request, notification.notification_uuid)
            progress_detail_list = []
            for progress_detail in notification_obj.recovery_workflow_details:
                # Retrieve progress name from detailed name.
                action = progress_detail['name']
                for task in progress_detail['progress_details']:
                    id = id + 1
                    progress_obj = notification_tab.ProgressDetailsItem(
                        id,
                        action,
                        task['timestamp'],
                        task['message']
                    )
                    progress_detail_list.append(progress_obj)
            return progress_detail_list
        except Exception as e:
            error_message = _("Failed to get progress details"
                              " for notification '%(notification)', Reason: "
                              "%(reason)") % (
                {'notification': notification.notification_uuid, 'reason': e})

            exceptions.handle(self.request, error_message)
            return []


class NotificationDetailTabs(tabs.DetailTabsGroup):
    slug = "notification_details"
    if api_versions.APIVersion(
            plugin.DEFAULT_HA_API_VERSION) >= api_versions.APIVersion("1.1"):
        tabs = (OverviewTab, NotificationProgressDetailsTab)
    else:
        tabs = (OverviewTab,)



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from django.urls import reverse

from masakaridashboard.test import helpers as test

INDEX_URL = reverse('horizon:masakaridashboard:hosts:index')


class HostTest(test.TestCase):

    def test_index(self):
        hosts = self.masakari_host.list()
        segments = self.masakari_segment.list()
        with mock.patch('masakaridashboard.api.api.segment_list',
                        return_value=segments), mock.patch(
                'masakaridashboard.api.api.get_segment',
                return_value=segments[0]), mock.patch(
                'masakaridashboard.api.api.get_host_list',
                return_value=hosts):
            res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'masakaridashboard/hosts/index.html')
        self.assertEqual(res.status_code, 200)

    def test_create_post(self):
        segment = self.masakari_segment.list()
        host = self.masakari_host.list()[0]
        hypervisors = self.hypervisors.list()
        create_url = reverse('horizon:masakaridashboard:segments:addhost',
                             args=[segment[0].uuid])
        form_data = {
            'segment_id': host.failover_segment_id,
            'segment_name': segment[0].name,
            'name': host.name,
            'type': host.type,
            'reserved': host.reserved,
            'control_attributes': host.control_attributes,
            'on_maintenance': host.on_maintenance
        }
        with mock.patch('masakaridashboard.api.api.segment_list',
                        return_value=segment), mock.patch(
                'masakaridashboard.api.api.get_host_list',
                return_value=[]), mock.patch(
                'masakaridashboard.api.api.get_hypervisor_list',
                return_value=hypervisors), mock.patch(
                'masakaridashboard.api.api.get_segment',
                return_value=segment[0]), mock.patch(
            'masakaridashboard.api.api.create_host',
                return_value=host) as mocked_create:
            res = self.client.post(create_url, form_data)
        self.assertNoFormErrors(res)
        self.assertEqual(res.status_code, 302)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        mocked_create.assert_called_once_with(
            mock.ANY,
            form_data
        )

    def test_delete_ok(self):
        host = self.masakari_host.list()[0]
        data = {'object_ids': host.uuid + ',' + host.failover_segment_id,
                'action': 'host__delete'}
        with mock.patch(
                'masakaridashboard.api.api.segment_list',
                return_value=[self.masakari_segment.first(
                )]), mock.patch(
                'masakaridashboard.api.api.get_host_list',
                return_value=self.masakari_host.list()), mock.patch(
                'masakaridashboard.api.api.delete_host',
                return_value=None
        ) as mocked_delete:
            res = self.client.post(INDEX_URL, data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        mocked_delete.assert_called_once_with(
            mock.ANY,
            host.uuid,
            host.failover_segment_id,
        )

    def test_detail(self):
        host = self.masakari_host.list()[0]
        id_to_update = host.uuid+','+host.failover_segment_id
        detail_url = reverse('horizon:masakaridashboard:hosts:detail',
                             args=[id_to_update])
        with mock.patch('masakaridashboard.api.api.get_host',
                        return_value=self.masakari_host.list()[0]):
            res = self.client.get(detail_url)
        self.assertNoFormErrors(res)
        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(res, 'horizon/common/_detail.html')
        self.assertTemplateUsed(
            res, 'masakaridashboard/hosts/_detail_overview.html')

    def test_update(self):
        host_to_update = self.masakari_host.list()[0]
        id_to_update = (
            host_to_update.uuid+','+host_to_update.failover_segment_id)
        update_url = reverse('horizon:masakaridashboard:hosts:update',
                             args=[id_to_update])
        host_to_update.control_attributes = 'fake'
        form_data = {
            'failover_segment_id': host_to_update.failover_segment_id,
            'uuid': host_to_update.uuid,
            'name': host_to_update.name,
            'type': host_to_update.type,
            'reserved': host_to_update.reserved,
            'control_attributes': host_to_update.control_attributes,
            'on_maintenance': host_to_update.on_maintenance
        }
        with mock.patch(
                'masakaridashboard.api.api.get_host',
                return_value=self.masakari_host.list()[0]), mock.patch(
                'masakaridashboard.api.api.update_host',
                return_value=host_to_update) as mocked_update:
            res = self.client.post(update_url, form_data)

        self.assertNoFormErrors(res)
        self.assertEqual(res.status_code, 302)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        fields_to_update = {
            'name': host_to_update.name,
            'type': host_to_update.type,
            'reserved': host_to_update.reserved,
            'control_attributes': host_to_update.control_attributes,
            'on_maintenance': host_to_update.on_maintenance
        }

        mocked_update.assert_called_once_with(
            mock.ANY,
            host_to_update.uuid,
            host_to_update.failover_segment_id,
            fields_to_update
        )



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from oslo_utils import timeutils

from openstack.instance_ha.v1 import host
from openstack.instance_ha.v1 import notification
from openstack.instance_ha.v1 import segment
from openstack_dashboard.test.test_data import utils as test_data_utils

from masakaridashboard.test import uuidsentinel
from novaclient.v2.hypervisors import Hypervisor
from novaclient.v2.hypervisors import HypervisorManager

NOW = timeutils.utcnow().replace(microsecond=0)


def data(TEST):

    TEST.masakari_segment = test_data_utils.TestDataContainer()

    segment1 = segment.Segment(uuid=uuidsentinel.segment1, name='test',
                               recovery_method='auto',
                               service_type='service', description='demo')
    segment2 = segment.Segment(uuid=uuidsentinel.segment2,
                               name='test2', recovery_method='auto',
                               service_type='service', description='demo')
    segment3 = segment.Segment(uuid=uuidsentinel.segment3, name='test3',
                               recovery_method='auto',
                               service_type='service', description='demo')

    TEST.masakari_segment.add(segment1)
    TEST.masakari_segment.add(segment2)
    TEST.masakari_segment.add(segment3)

    TEST.masakari_host = test_data_utils.TestDataContainer()

    host1 = host.Host(uuid=uuidsentinel.host1, name="test",
                      reserved='True', type='service',
                      control_attributes='test',
                      failover_segment_id=uuidsentinel.segment1,
                      on_maintenance='False')

    TEST.masakari_host.add(host1)

    TEST.hypervisors = test_data_utils.TestDataContainer()

    hypervisor1 = Hypervisor(
        HypervisorManager, {'id':  '1', 'hypervisor_hostname': "test"})

    TEST.hypervisors.add(hypervisor1)

    TEST.masakari_notification = test_data_utils.TestDataContainer()
    notification1 = notification.Notification(
        notification_uuid=uuidsentinel.notification1, status='new',
        generated_time=(NOW - datetime.timedelta(seconds=2)),
        payload='test', type='type1', source_host_uuid=uuidsentinel.host1)
    notification2 = notification.Notification(
        notification_uuid=uuidsentinel.notification2, status='running',
        generated_time=(NOW - datetime.timedelta(seconds=3)),
        payload='test', type='type2', source_host_uuid=uuidsentinel.host2)
    notification3 = notification.Notification(
        notification_uuid=uuidsentinel.notification3, status='error',
        generated_time=(NOW - datetime.timedelta(seconds=4)),
        payload='test', type='type3', source_host_uuid=uuidsentinel.host3)
    TEST.masakari_notification.add(notification1)
    TEST.masakari_notification.add(notification2)
    TEST.masakari_notification.add(notification3)
